<?php
/**
 * 
 * @package 	personaldata
 * @version 	$Id: activities.php 1 2010-02-19$
 * @copyright 	PrimeLife Project, TU Dresden, Lehrstuhl Datenschutz und Datensicherheit
 * @author 		richard.wolsch@gmx.de
 * @license 	http://opensource.org/licenses/gpl-license.php GNU Public License
 *
 */
 
	/**
	 * @ignore
	 */
	if (!defined('IN_PHPBB'))
	{
		exit;
	}

	/**
	 * The last $count activities of the current user.
	 * @param int $count number of activities
	 * @return array activities
	 */
	function priv_get_latest_activities($count)
	{
		global $db, $user;
		$activities = array();
				
		// database query
		$actions_t		= 'phpbb_user_actions';
		$actiontypes_t	= 'phpbb_user_actiontypes';				
		
		$sql_array = array(
			'SELECT'		=> 'q.lang_id, q.privacy, p.action_time, p.arg_1, p.arg_2, p.arg_3, p.arg_4, p.arg_5, p.arg_6',
			'FROM'			=> array(
				$actions_t		=> 'p',
			),
			'LEFT_JOIN'		=> array(
				array(
					'FROM'		=> array(
						$actiontypes_t	=> 'q'
					),
					'ON'		=> 'p.action_type = q.action_type'
				)
			),
			'WHERE'			=> 'p.user_id = '.$user->data['user_id'],
			'ORDER_BY'		=> 'p.action_time DESC'
		);			
		$sql = $db->sql_build_query('SELECT', $sql_array);			
					
		$result = $db->sql_query_limit($sql, $count, 0);
		
		// generate result from query response
		while($row = $db->sql_fetchrow($result)){
			$activity = array(
				'DATE'			=> $row['action_time'],
				'ID'			=> mb_strtoupper($row['lang_id'], 'UTF-8'),
				'VISIBILITY'	=> mb_strtolower($row['privacy'], 'UTF-8')
			);
			for($j=1; $j<=6; $j++){
				$activity['ARG'.$j]	= $row['arg_'.$j];
			}
			$activities[] = $activity;			
		}			
		
		return $activities;
	}
	
	/**
	 * A random activity of the current user.
	 * @param int $min minimum index of this activity (use this to avoid duplicates with latest_activities)
	 * @return array activity
	 */
	function priv_get_rand_activity($min){		
		global $db, $user;

		// set default
		$activity = array(
			'SHOW'			=> false,
			'DATE'			=> 0,
			'ID'			=> 'UNKNOWN',
			'VISIBILITY'	=> 'safe'
		);

		// database queries
		$actions_t		= 'phpbb_user_actions';
		$actiontypes_t	= 'phpbb_user_actiontypes';				

		// count all actions of this user
		$sql_array_1 = array(
			'SELECT'		=> 'count(p.action_time) as action_count',
			'FROM'			=> array(
				$actions_t		=> 'p'
			),
			'WHERE'			=> 'p.user_id = '.$user->data['user_id']
		);
		$sql_1 = $db->sql_build_query('SELECT', $sql_array_1);			
		$result_1 = $db->sql_query($sql_1);

		// if not enaugh actions available, then leave
		$count = $db->sql_fetchfield('action_count', 0, $result_1);
		if($count < $min)
			return $activity;
		
		// query random action
		$rand = rand($min-1, $count-1);
		
		$sql_array_2 = array(
			'SELECT'		=> 'q.lang_id, q.privacy, p.action_time, p.arg_1, p.arg_2, p.arg_3, p.arg_4, p.arg_5, p.arg_6',
			'FROM'			=> array(
				$actions_t		=> 'p',
			),
			'LEFT_JOIN'		=> array(
				array(
					'FROM'		=> array(
						$actiontypes_t	=> 'q'
					),
					'ON'		=> 'p.action_type = q.action_type'
				)
			),
			'WHERE'			=> 'p.user_id = '.$user->data['user_id'],
			'ORDER_BY'		=> 'p.action_time DESC'
		);			
		$sql_2 = $db->sql_build_query('SELECT', $sql_array_2);			
		$result_2 = $db->sql_query_limit($sql_2, 1, $rand);
		
		// generate result from query response		
		$activity = array(
			'SHOW'			=> true,
			'DATE'			=> $db->sql_fetchfield('action_time', 0, $result_2),
			'ID'			=> mb_strtoupper($db->sql_fetchfield('lang_id', 0, $result_2), 'UTF-8'),
			'VISIBILITY'	=> mb_strtolower($db->sql_fetchfield('privacy', 0, $result_2), 'UTF-8')
		);
		for($j=1; $j<=6; $j++){
			$activity['ARG'.$j]	= $db->sql_fetchfield('arg_'.$j, 0, $result_2);
		}
				
		return $activity;
	}
	
	/**
	 * Is “latest activities panel” enabled?
	 * @return bool enabled
	 */
	function priv_get_show_activities(){
		global $user;
		return $user->data['user_id'] != 1;
	}
	
	/**
	 * Write a new action to the database.
	 * @param string $type action type
	 * (e.g. 'TOPIC_READ', 'TOPIC_WRITE'
	 * – see db table 'phpbb_user_actiontypes' for details)
	 * @param array $args arguments used in the language file as 'ARG1', 'ARG2', … (maximum is 6)
	 * @return bool success (false, if $type was not valid)
	 */
	function priv_register_activity($type, $args){
		global $user, $db;
		
		// database queries
		$actions_t		= 'phpbb_user_actions';
		$actiontypes_t	= 'phpbb_user_actiontypes';
		
		// get latest action of current user and compare it
		// (avoid duplicate action on reload and action schemes)
		$sql_array_1 = array(
			'SELECT'		=> 'p.action_id, q.lang_id, p.arg_1, p.arg_2, p.arg_3, p.arg_4, p.arg_5, p.arg_6',
			'FROM'			=> array(
				$actions_t		=> 'p'
			),
			'LEFT_JOIN'		=> array(
				array(
					'FROM'		=> array(
						$actiontypes_t	=> 'q'
					),
					'ON'		=> 'p.action_type = q.action_type'
				)
			),
			'WHERE'			=> 'p.user_id = '.$user->data['user_id'],
			'ORDER_BY'		=> 'p.action_time DESC'
		);
		$sql_1 = $db->sql_build_query('SELECT', $sql_array_1);			
		$result_1 = $db->sql_query_limit($sql_1, 1, 0);
		
		$last_action_id = $db->sql_fetchfield('action_id', 0, $result_1);
		$last_lang_id 	= $db->sql_fetchfield('lang_id', 0, $result_1);
		$last_arg_1 	= $db->sql_fetchfield('arg_1', 0, $result_1);
		$last_arg_2 	= $db->sql_fetchfield('arg_2', 0, $result_1);
		$usedtype		= $type;
		$last_remove	= false;

		if(!empty($last_lang_id)){
			if(($type == $last_lang_id) 
				&& ($args[0] == $last_arg_1) 
				&& ($args[1] == $last_arg_2))
				return false;						// avoid duplicate entries
				
			if(($type == 'TOPIC_WRITE_ALL') 
				&& ($last_lang_id == 'TOPIC_READ')
				&& ($args[0] == $last_arg_1) 
				&& ($args[1] == $last_arg_2)){
				$usedtype 		= 'TOPIC_RW_ALL';	// topic read followed by topic write
				$last_remove 	= true;
			}

			if(($type == 'TOPIC_WRITE_GROUP') 
				&& ($last_lang_id == 'TOPIC_READ')
				&& ($args[0] == $last_arg_1) 
				&& ($args[1] == $last_arg_2)){
				$usedtype 		= 'TOPIC_RW_GROUP';	// topic read followed by topic write
				$last_remove	= true;
			}

			if(($type == 'TOPIC_READ') 
				&& (($last_lang_id == 'TOPIC_WRITE_ALL') || ($last_lang_id == 'TOPIC_RW_ALL')
					||($last_lang_id == 'TOPIC_WRITE_GROUP') || ($last_lang_id == 'TOPIC_RW_GROUP'))
				&& ($args[0] == $last_arg_1) 
				&& ($args[1] == $last_arg_2))
				return false;						// reading topic after posting
			
			if(($type == 'PRIVATE_MESSAGE_READ') 
				&& ($last_lang_id == 'PRIVATE_MESSAGE_WRITE')
				&& ($args[1] == $last_arg_2)) 
				return false;						// reading PM after writing it
		}

		// remove last action (if substituted by new one)
		if($last_remove){
			$sql4 = 'DELETE FROM '.$actions_t.' WHERE action_id = '.$last_action_id;
			$db->sql_query($sql4);
		}

		// get type identifier
		$sql_array_2 = array(
			'SELECT'		=> 'p.action_type',
			'FROM'			=> array(
				$actiontypes_t	=> 'p'
			),
			'WHERE'			=> "p.lang_id = '$usedtype'"
		);
		$sql_2 = $db->sql_build_query('SELECT', $sql_array_2);			
		$result_2 = $db->sql_query($sql_2);
		
		if(count($db->sql_fetchrowset($result_2)) < 1)
			return false;
		
		$action_type = $db->sql_fetchfield('action_type', 0, $result_2);
				
		// insert data
		$sql_array_3 = array(
			'user_id'		=> $user->data['user_id'],
			'action_type'	=> $action_type,
			'action_time'	=> time()
		);
		for($i=1; $i<=6; $i++){
			if(isset($args[$i-1]))
				$sql_array_3['arg_'.$i] = $args[$i-1];
		}
		$sql_3 = 'INSERT INTO '.$actions_t.' '.$db->sql_build_array('INSERT', $sql_array_3);
		$db->sql_query($sql_3);
		
		return true;			
	}
	
	/**
	 * return access rights of fourm
	 * @param integer $forum_id Identifier of forum
	 * @return string access rights
	 */
	function priv_get_forum_rights($forum_id){
		global $db;
		
		// database queries
		$groups_t		= 'phpbb_acl_groups';
		
		// get forum access rights
		$sql_array = array(
			'SELECT'		=> 'p.forum_id',
			'FROM'			=> array(
				$groups_t	=> 'p'
			),
			'WHERE'			=> "p.forum_id=$forum_id AND p.group_id=1"
		);
		$sql = $db->sql_build_query('SELECT', $sql_array);			
		$result = $db->sql_query($sql);
		
		if(count($db->sql_fetchrowset($result)) < 1)
			return 'GROUP';
		return 'ALL';
	}
?>