<?php
/**
 * 
 * @package 	personaldata
 * @version 	$Id: location.php 1 2010-02-19$
 * @copyright 	PrimeLife Project, TU Dresden, Lehrstuhl Datenschutz und Datensicherheit
 * @author 		richard.wolsch@gmx.de
 * @license 	http://opensource.org/licenses/gpl-license.php GNU Public License
 *
 */
 
	/**
	 * @ignore
	 */
	if (!defined('IN_PHPBB'))
	{
		exit;
	}
  
	/**
	 * Fills an SimpleXMLElement with all data about an ip-address known from
	 * ipinfodb.com
	 * @param string ip_address
	 * @return SimpleXMLElement ipinfodb data
	 */
  	function priv_get_ipinfodb($ip_address){
  		$content 	= file_get_contents('http://ipinfodb.com/ip_query.php?ip='.$ip_address.'&output=xml&timezone=false');
  		if (!isset($content))
    		$content = file_get_contents('http://backup.ipinfodb.com/ip_query.php?ip='.$ip_address.'&output=xml&timezone=false');  		
  		$answer 	= simplexml_load_string($content);
  		return $answer;
  	}

	/**
	 * Fills an array with some data about an ip-address known from
	 * whois call
	 * @param string ip_address
	 * @return array whois data
	 */
	function priv_get_whois($ip_address){
	    $data = array();
	
	    // Internet Assigned Numbers Authority
	    // Which whois-server has the IP-address?
	    $root_server    = array('whois.iana.org', '43');
	    $whois_server   = array('', '43');
	    $server_connection = @fsockopen($root_server[0], $root_server[1]);    
	
	    if($server_connection){
	        // connection success
	        fputs($server_connection, "$ip_address\r\n");
	        while(!feof($server_connection)){
	            $property = fgets($server_connection, 512);
	            if(strpos(strtolower($property), 'whois:') === 0)
	                $whois_server[0] = trim(substr($property, 6));
	        }
	        fclose($server_connection); 
	    }
	
	    // Regional Internet Registry
	    // Who registered this IP-address?
	    $server_connection = @fsockopen($whois_server[0], $whois_server[1]);    
	
	    if($server_connection){
	        // connection success
	        fputs($server_connection, "$ip_address\r\n");
	        while(!feof($server_connection)){
	            $property = fgets($server_connection, 512);
	            
	            if(!empty($property)){
	                // title
	                if(((strpos(strtolower($property), 'orgname') === 0) ||
	                    (strpos(strtolower($property), 'org-name') === 0) ||
	                    (strpos(strtolower($property), 'descr') === 0))
	                    && (!isset($data['title'])))
	                    $data['title'] = trim(substr($property, strpos($property, ':') + 1));
	                // net
	                if(((strpos(strtolower($property), 'netname') === 0))
	                    && (!isset($data['net'])))
	                    $data['net'] = trim(substr($property, strpos($property, ':') + 1));
	                // city
	                if(((strpos(strtolower($property), 'city') === 0))
	                    && (!isset($data['city'])))
	                    $data['city'] = trim(substr($property, strpos($property, ':') + 1));
	                // country
	                if(((strpos(strtolower($property), 'country') === 0))
	                    && (!isset($data['country'])))
	                    $data['country'] = trim(substr($property, strpos($property, ':') + 1));
	            }                
	        }
	        fclose($server_connection); 
	    }
		return $data;
	}

	/**
	 * Retrieves country, city and profider from the ip-address.
	 * @return array location
	 */
	function priv_get_location()
	{
		global $user;
		$ip_address = $_SERVER['REMOTE_ADDR'];		

		// TODO: uncomment
		//$ip_address = '78.55.161.99';	// for testing on localhost
		
		if(empty($ip_address) || ($ip_address == '127.0.0.1') || ($ip_address == '::1')){
			// using your phpBB locally
			$location['FLAG']		= '';
			$location['CITY']		= 'Localhost-City ;-)';
			$location['COUNTRY']	= '';
			$location['PROVIDER']	= $user->lang['PRIV_UNKNOWN'];
			return $location;
		}
		
		$ipinfodb = priv_get_ipinfodb($ip_address);
		$whois = priv_get_whois($ip_address);
		
		$location = array();
		$location['FLAG']		= 'http://api.hostip.info/flag.php?ip='.$ip_address;
		$location['CITY']		= empty($ipinfodb->City) ? $user->lang['PRIV_LOCATION_UNKNOWN'] : $ipinfodb->City;
		if(!empty($ipinfodb->RegionName))
    		$location['CITY']   .= ' ('.$ipinfodb->RegionName.')';
		// if there is notranslation available for the country name, this will be shown in English
		$location['COUNTRY']	= (isset($user->lang['PRIV_'.mb_strtoupper($ipinfodb->CountryName, 'UTF-8')])) ? $user->lang['PRIV_'.mb_strtoupper($ipinfodb->CountryName, 'UTF-8')] : $ipinfodb->CountryName;
		if(!empty($whois['title']))
		    $location['PROVIDER']	= $whois['title'];
		else {
	    	$location['PROVIDER']   = $user->lang['PRIV_UNKNOWN'];
		}
		if(!empty($whois['net']))
            $location['PROVIDER']   .= ' ('.$whois['net'].')';
		return $location;
	}
	
?>
