<?php
/**
 * 
 * @package 	personaldata
 * @version 	$Id: profile.php 1 2010-02-19$
 * @copyright 	PrimeLife Project, TU Dresden, Lehrstuhl Datenschutz und Datensicherheit
 * @author 		richard.wolsch@gmx.de
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 *
 */
 
	/**
	 * @ignore
	 */
	if (!defined('IN_PHPBB'))
	{
		exit;
	}
 
	/**
	 * Search position of user datetime pattern that
	 * separates date from time.
	 * @return array positions: start of date, end of date, start of time, end of time
	 */
	function priv_get_datetime_pattern_sep()
	{
		global $user;
		$datevars = array('d', 'D', 'j', 'l', 'N', 'S', 'w', 'z', 'W', 'F', 'm',
		 	'M', 'n', 't', 'L', 'o', 'Y', 'y');
		$timevars = array('a', 'A', 'B', 'g', 'G', 'h', 'H', 'i', 's', 'u', 'e',
		 	'I', 'O', 'P', 'T', 'Z');
		$result = array(100,0,100,0);
		foreach($datevars as $value){
			$pos = strpos($user->data['user_dateformat'], $value);
			if($pos === false)
				continue;
			if($pos<$result[0])
				$result[0] = $pos;
			if($pos>$result[1])
				$result[1] = $pos;
		}
		foreach($timevars as $value){
			$pos = strpos($user->data['user_dateformat'], $value);
			if($pos === false)
				continue;
			if($pos<$result[2])
				$result[2] = $pos;
			if($pos>$result[3])
				$result[3] = $pos;
		}
		return $result;
	}

	/**
	 * User defined time pattern
	 * if pattern is “d.M.Y, H:i” this will return “H:i”  
	 * @return string time pattern
	 */
	function priv_get_time_pattern()
	{		
		global $user;
		$pos = priv_get_datetime_pattern_sep();
		return trim(substr($user->data['user_dateformat'], $pos[2], $pos[3]-$pos[2]+1));
	}
	
	/**
	 * User defined date pattern
	 * if pattern is “d.M.Y, H:i” this will return “d.M.Y”  
	 * @return string date pattern
	 */
	function priv_get_date_pattern()
	{
		global $user;
		$pos = priv_get_datetime_pattern_sep();
		return trim(substr($user->data['user_dateformat'], $pos[0], $pos[1]-$pos[0]+1));
	}
	
	/**
	 * Chooses a random profile information
	 * @return array profile data
	 */
	function priv_get_rand_profile()
	{
		global $user;
		$profile = array();

		// set default result
		$profile['KEY'] 		= 'unknown';
		$profile['VALUE'] 		= 'unknown';
		$profile['VISIBILITY']	= 'safe';
		$profile['LINK']		= append_sid('ucp.php?i=profile', false, true, $user->session_id);
		$profile['SHOW']		= false;
		
		// guest
		if($user->data['user_id'] == 1)
			return $profile;
		else
			$profile['SHOW']	= true;
			
		$date_pattern = priv_get_date_pattern();
		$time_pattern = priv_get_time_pattern();

		// data list
		$profilestrings = array(
			'REGISTRATION_DATE'		=> array(str_replace(" ", "&nbsp;", date($date_pattern, $user->data['user_regdate'])), 'danger'),
			'SESSION_START_DATE'	=> array(str_replace(" ", "&nbsp;", date($time_pattern, $user->data['session_start'])), 'safe'),
			'NUMBER_OF_POSTS'		=> array($user->data['user_posts'], 'danger'),
			'ONLINE_STATE'			=> array($user->data['user_allow_viewonline'] ? $user->lang['PRIV_PROFILE_VISIBLE'] : $user->lang['PRIV_PROFILE_INVISIBLE'],
											$user->data['user_allow_viewonline'] ? 'danger' : 'safe')
		);
	
		// add some more data, if it exists
		if(!empty($user->data['user_email']))
			$profilestrings['EMAIL'] = array($user->data['user_email'], $user->data['user_allow_viewemail'] ? 'medium' : 'safe');
		if(!empty($user->data['user_lastvisit']))
			$profilestrings['LAST_VISIT_DATE'] = array(str_replace(" ", "&nbsp;", date($date_pattern, $user->data['user_lastvisit'])), 'medium');
		if(!empty($user->data['user_lastpost_time']))
			$profilestrings['LAST_POST_DATE'] = array(str_replace(" ", "&nbsp;", date($date_pattern, $user->data['user_lastpost_time'])), 'medium');
		if(!empty($user->data['user_birthday'])){
			list($day, $month, $year) = explode('-', $user->data['user_birthday']);
			$profilestrings['BIRTHDAY'] = array(str_replace(" ", "&nbsp;", date($date_pattern, mktime(0, 0, 0, $month, $day, $year))), 'medium'); 
		}
		if(!empty($user->data['user_from']))
			$profilestrings['LOCATION']	= array($user->data['user_from'], 'danger');
		if(!empty($user->data['user_icq']))
			$profilestrings['ICQ']	= array($user->data['user_icq'], 'danger');
		if(!empty($user->data['user_aim']))
			$profilestrings['AIM']	= array($user->data['user_aim'], 'medium');
		if(!empty($user->data['user_yim']))
			$profilestrings['YIM']	= array($user->data['user_yim'], 'danger');
		if(!empty($user->data['user_msnm']))
			$profilestrings['MSNM']	= array($user->data['user_msnm'], 'medium');
		if(!empty($user->data['user_jabber']))
			$profilestrings['JABBER']	= array($user->data['user_jabber'], 'medium');
		if(!empty($user->data['user_website']))
			$profilestrings['WEBSITE']	= array($user->data['user_website'], 'danger');
		if(!empty($user->data['user_occ']))
			$profilestrings['OCCUPATION']	= array($user->data['user_occ'], 'medium');
		if(!empty($user->data['user_interests']))
			$profilestrings['INTERESTS']	= array($user->data['user_interests'], 'medium');
			
		// choose random data from list
		$rand = rand(0, count($profilestrings)-1);
		$i = 0;
		foreach($profilestrings as $key => $value){			
			if($i == $rand){
				$profile['KEY']			= $key;
				$profile['VALUE']		= $value[0];
				$profile['VISIBILITY']	= $value[1];
        		$profile['LINK']		= append_sid('ucp.php?i=profile', false, true, $user->session_id);
				break;
			}
			$i++;
		}
		return $profile;
	}
?>
